import * as cf from '@aws-cdk/aws-cloudfront';
import * as lambda from '@aws-cdk/aws-lambda';
import * as cdk from '@aws-cdk/core';
import { ServerlessApp } from './';
/**
 * The Extension interface.
 *
 * @stability stable
 */
export interface IExtensions {
    /**
     * Lambda function ARN for this extension.
     *
     * @stability stable
     */
    readonly functionArn: string;
    /**
     * Lambda function version for the function.
     *
     * @stability stable
     */
    readonly functionVersion: lambda.Version;
    /**
     * The Lambda edge event type for this extension.
     *
     * @stability stable
     */
    readonly eventType: cf.LambdaEdgeEventType;
}
/**
 * The modify response header extension.
 *
 * @see https://console.aws.amazon.com/lambda/home#/create/app?applicationId=arn:aws:serverlessrepo:us-east-1:418289889111:applications/modify-response-header
 * @stability stable
 */
export declare class ModifyResponseHeader extends ServerlessApp implements IExtensions {
    /**
     * Lambda function ARN for this extension.
     *
     * @stability stable
     */
    readonly functionArn: string;
    /**
     * Lambda function version for the function.
     *
     * @stability stable
     */
    readonly functionVersion: lambda.Version;
    /**
     * The Lambda edge event type for this extension.
     *
     * @stability stable
     */
    readonly eventType: cf.LambdaEdgeEventType;
    /**
     * @stability stable
     */
    constructor(scope: cdk.Construct, id: string);
}
/**
 * Construct properties for AntiHotlinking.
 *
 * @stability stable
 */
export interface AntiHotlinkingProps {
    /**
     * Referer allow list with wildcard(* and ?) support i.e. `example.com` or `exa?ple.*`.
     *
     * @stability stable
     */
    readonly referer: string[];
}
/**
 * The Anti-Hotlinking extension.
 *
 * @see https://console.aws.amazon.com/lambda/home#/create/app?applicationId=arn:aws:serverlessrepo:us-east-1:418289889111:applications/anti-hotlinking
 * @stability stable
 */
export declare class AntiHotlinking extends ServerlessApp implements IExtensions {
    /**
     * Lambda function ARN for this extension.
     *
     * @stability stable
     */
    readonly functionArn: string;
    /**
     * Lambda function version for the function.
     *
     * @stability stable
     */
    readonly functionVersion: lambda.Version;
    /**
     * The Lambda edge event type for this extension.
     *
     * @stability stable
     */
    readonly eventType: cf.LambdaEdgeEventType;
    /**
     * @stability stable
     */
    constructor(scope: cdk.Construct, id: string, props: AntiHotlinkingProps);
}
/**
 * Security Headers extension.
 *
 * @see https://aws.amazon.com/tw/blogs/networking-and-content-delivery/adding-http-security-headers-using-lambdaedge-and-amazon-cloudfront/
 * @stability stable
 */
export declare class SecurtyHeaders extends ServerlessApp implements IExtensions {
    /**
     * Lambda function ARN for this extension.
     *
     * @stability stable
     */
    readonly functionArn: string;
    /**
     * Lambda function version for the function.
     *
     * @stability stable
     */
    readonly functionVersion: lambda.Version;
    /**
     * The Lambda edge event type for this extension.
     *
     * @stability stable
     */
    readonly eventType: cf.LambdaEdgeEventType;
    /**
     * @stability stable
     */
    constructor(scope: cdk.Construct, id: string);
}
/**
 * Custom extension sample.
 *
 * @stability stable
 */
export declare class Custom implements IExtensions {
    /**
     * Lambda function ARN for this extension.
     *
     * @stability stable
     */
    readonly functionArn: string;
    /**
     * Lambda function version for the function.
     *
     * @stability stable
     */
    readonly functionVersion: lambda.Version;
    /**
     * The Lambda edge event type for this extension.
     *
     * @stability stable
     */
    readonly eventType: cf.LambdaEdgeEventType;
    /**
     * @stability stable
     */
    constructor(scope: cdk.Construct, id: string);
}
